@@ -86,7 +86,7 @@ static void https_tls_write(const uint8_t *data, size_t len, void *user_data)
 	int r;
 
 	while (len) {
-		r = write(l_io_get_fd(io), data, len);
+		r = send(l_io_get_fd(io), data, len, MSG_NOSIGNAL);
 		if (r < 0) {
 			l_main_quit();
 			break;
@@ -115,7 +115,7 @@ static void https_tls_ready(const char *peer_identity, void *user_data)
 
 static void https_tls_debug_cb(const char *str, void *user_data)
 {
-	l_info("%s", str);
+	printf("%s\n", str);
 }
 
 int main(int argc, char *argv[])
@@ -124,7 +124,7 @@ int main(int argc, char *argv[])
 	struct in_addr **addr_list;
 	struct sockaddr_in addr;
 	int fd;
-	bool auth_ok = true;
+	bool auth_ok;
 
 	if (argc != 2 && argc != 3 && argc != 6) {
 		printf("Usage: %s <https-host-name> [<ca-cert-path> "
@@ -180,13 +180,14 @@ int main(int argc, char *argv[])
 	if (getenv("TLS_DEBUG"))
 		l_tls_set_debug(tls, https_tls_debug_cb, NULL, NULL);
 
-	if (argc > 2)
-		l_tls_set_cacert(tls, argv[2]);
-	if (argc > 5)
-		auth_ok = l_tls_set_auth_data(tls, argv[3], argv[4], argv[5]);
+	auth_ok = (argc <= 2 || l_tls_set_cacert(tls, argv[2])) &&
+		(argc <= 5 ||
+		 l_tls_set_auth_data(tls, argv[3], argv[4], argv[5]));
 
 	if (tls && auth_ok)
 		l_main_run();
+	else
+		printf("TLS setup failed\n");
 
 	l_io_destroy(io);
 	l_tls_free(tls);
