@@ -52,9 +52,11 @@ static bool https_io_read(struct l_io *io, void *user_data)
 	int l;
 
 	l = read(l_io_get_fd(io), buf, sizeof(buf));
-	if (l == 0)
+	if (l == 0) {
+		if (!served)
+			printf("EOF before serving a page\n");
 		l_main_quit();
-	else if (l > 0)
+	} else if (l > 0)
 		l_tls_handle_rx(tls, buf, l);
 
 	return true;
@@ -78,9 +80,9 @@ static void https_new_data(const uint8_t *data, size_t len, void *user_data)
 
 	if (len >= 4 && !memcmp(data + len - 4, "\r\n\r\n", 4)) {
 		l_tls_write(tls, (void *) reply, strlen(reply));
-		l_tls_close(tls);
 		served = true;
 		printf("Hello world page served\n");
+		l_tls_close(tls);
 	}
 }
 
@@ -89,8 +91,9 @@ static void https_tls_write(const uint8_t *data, size_t len, void *user_data)
 	int r;
 
 	while (len) {
-		r = write(l_io_get_fd(io), data, len);
+		r = send(l_io_get_fd(io), data, len, MSG_NOSIGNAL);
 		if (r < 0) {
+			printf("send error\n");
 			l_main_quit();
 			break;
 		}
@@ -109,7 +112,7 @@ static void https_tls_ready(const char *peer_identity, void *user_data)
 
 static void https_tls_debug_cb(const char *str, void *user_data)
 {
-	l_info("%s", str);
+	printf("%s\n", str);
 }
 
 int main(int argc, char *argv[])
@@ -171,11 +174,13 @@ int main(int argc, char *argv[])
 	if (getenv("TLS_DEBUG"))
 		l_tls_set_debug(tls, https_tls_debug_cb, NULL, NULL);
 
-	auth_ok = l_tls_set_auth_data(tls, argv[1], argv[2], argv[3]);
-	l_tls_set_cacert(tls, argc > 4 ? argv[4] : NULL);
+	auth_ok = l_tls_set_auth_data(tls, argv[1], argv[2], argv[3]) &&
+		(argc <= 4 || l_tls_set_cacert(tls, argv[4]));
 
-	if (tls && !auth_ok)
+	if (tls && auth_ok)
 		l_main_run();
+	else
+		printf("TLS setup failed\n");
 
 	l_io_destroy(io);
 	l_tls_free(tls);
