@@ -28,6 +28,8 @@
 
 #include <ell/ell.h>
 
+#include "ell/tls-private.h"
+
 #define KEY1_STR "This key has exactly _32_ bytes!"
 #define KEY1_LEN (strlen(KEY1_STR))
 #define KEY2_STR "This key is longer than 32 bytes, just to be different."
@@ -394,23 +396,20 @@ static void test_trusted_keyring(const void *data)
 {
 	struct l_keyring *ring;
 	struct l_keyring *trust;
-	uint8_t *cacert;
-	size_t cacertlen;
-	uint8_t *cert;
-	size_t certlen;
+	struct l_cert *cacert;
+	struct l_cert *cert;
 	struct l_key *cakey;
 	struct l_key *key;
 	bool success;
 
-	cacert = l_pem_load_certificate(CERTDIR "cert-ca.pem", &cacertlen);
+	cacert = tls_cert_load_file(CERTDIR "cert-ca.pem");
 	assert(cacert);
-	cert = l_pem_load_certificate(CERTDIR "cert-server.pem",
-					&certlen);
+	cert = tls_cert_load_file(CERTDIR "cert-server.pem");
 	assert(cert);
 
-	cakey = l_key_new(L_KEY_RSA, cacert, cacertlen);
+	cakey = l_cert_get_pubkey(cacert);
 	assert(cakey);
-	key = l_key_new(L_KEY_RSA, cert, certlen);
+	key = l_cert_get_pubkey(cert);
 	assert(key);
 
 	trust = l_keyring_new();
@@ -431,43 +430,41 @@ static void test_trusted_keyring(const void *data)
 	l_keyring_free(ring);
 	l_key_free(cakey);
 	l_key_free(key);
-	l_free(cacert);
-	l_free(cert);
+	l_cert_free(cacert);
+	l_cert_free(cert);
 }
 
 static void test_trust_chain(const void *data)
 {
 	struct l_keyring *ring;
 	struct l_keyring *trust;
-	uint8_t *cacert;
-	size_t cacertlen;
-	uint8_t *intcert;
-	size_t intcertlen;
-	uint8_t *cert;
-	size_t certlen;
+	struct l_keyring *trust2;
+	struct l_cert *cacert;
+	struct l_cert *intcert;
+	struct l_cert *cert;
 	struct l_key *cakey;
 	struct l_key *intkey;
 	struct l_key *key;
 	bool success;
 
-	cacert = l_pem_load_certificate(CERTDIR "cert-ca.pem", &cacertlen);
+	cacert = tls_cert_load_file(CERTDIR "cert-ca.pem");
 	assert(cacert);
-	intcert = l_pem_load_certificate(CERTDIR "cert-intca.pem",
-						&intcertlen);
+	intcert = tls_cert_load_file(CERTDIR "cert-intca.pem");
 	assert(intcert);
-	cert = l_pem_load_certificate(CERTDIR "cert-entity-int.pem",
-					&certlen);
+	cert = tls_cert_load_file(CERTDIR "cert-entity-int.pem");
 	assert(cert);
 
-	cakey = l_key_new(L_KEY_RSA, cacert, cacertlen);
+	cakey = l_cert_get_pubkey(cacert);
 	assert(cakey);
-	intkey = l_key_new(L_KEY_RSA, intcert, intcertlen);
+	intkey = l_cert_get_pubkey(intcert);
 	assert(intkey);
-	key = l_key_new(L_KEY_RSA, cert, certlen);
+	key = l_cert_get_pubkey(cert);
 	assert(key);
 
 	trust = l_keyring_new();
 	assert(trust);
+	trust2 = l_keyring_new();
+	assert(trust2);
 	ring = l_keyring_new();
 	assert(ring);
 
@@ -486,24 +483,56 @@ static void test_trust_chain(const void *data)
 	assert(success);
 	success = l_keyring_link(ring, key);
 	assert(success);
+	success = l_keyring_link_nested(ring, trust);
+	assert(!success);
+	success = l_keyring_link_nested(ring, trust2);
+	assert(!success);
+	success = l_keyring_link_nested(ring, ring);
+	assert(!success);
+	success = l_keyring_unlink(ring, key);
+	assert(success);
+	success = l_keyring_unlink(ring, intkey);
+	assert(success);
+	l_key_free(cakey);
+	success = l_keyring_unlink(ring, intkey);
+	assert(!success);
+	success = l_keyring_link_nested(trust, trust2);
+	assert(success);
+	success = l_keyring_link(ring, key);
+	assert(!success);
+	success = l_keyring_link(trust2, intkey);
+	assert(success);
+	success = l_keyring_link(ring, key);
+	assert(success);
+	success = l_keyring_unlink(ring, key);
+	assert(success);
+	success = l_keyring_unlink_nested(trust, trust2);
+	assert(success);
+	success = l_keyring_link(ring, key);
+	assert(!success);
+	success = l_keyring_link_nested(trust, trust2);
+	assert(success);
+	l_keyring_free_norevoke(trust2);
+	success = l_keyring_link(ring, key);
+	assert(success);
+	success = l_keyring_unlink(ring, key);
+	assert(success);
+	l_key_free_norevoke(intkey);
+	success = l_keyring_link(ring, key);
+	assert(success);
 
 	l_keyring_free(trust);
 	l_keyring_free(ring);
-	l_key_free(cakey);
-	l_key_free(intkey);
 	l_key_free(key);
-	l_free(cacert);
-	l_free(intcert);
-	l_free(cert);
+	l_cert_free(cacert);
+	l_cert_free(intcert);
+	l_cert_free(cert);
 }
 
 static void test_key_crypto(const void *data)
 {
-	uint8_t *cert;
-	size_t certlen;
-	uint8_t *pubcert;
-	size_t pubcertlen;
-	struct l_key *key;
+	struct l_cert *cert;
+	struct l_key *privkey;
 	struct l_key *pubkey;
 	bool is_public;
 	size_t keybits;
@@ -514,24 +543,21 @@ static void test_key_crypto(const void *data)
 	int hash = L_CHECKSUM_NONE;
 	int rsa = L_KEY_RSA_PKCS1_V1_5;
 
-	cert = l_pem_load_private_key(CERTDIR "cert-client-key-pkcs8.pem",
-					NULL, NULL, &certlen);
+	cert = tls_cert_load_file(CERTDIR "cert-client.pem");
 	assert(cert);
-	pubcert = l_pem_load_certificate(CERTDIR "cert-client.pem",
-						&pubcertlen);
-	assert(pubcert);
-
-	key = l_key_new(L_KEY_RSA, cert, certlen);
-	assert(key);
-	pubkey = l_key_new(L_KEY_RSA, pubcert, pubcertlen);
+	pubkey = l_cert_get_pubkey(cert);
 	assert(pubkey);
+	l_cert_free(cert);
 
-	success = l_key_get_info(key, rsa, hash, &keybits, &is_public);
+	privkey = l_pem_load_private_key(CERTDIR "cert-client-key-pkcs8.pem",
+						NULL, NULL);
+	assert(privkey);
+	success = l_key_get_info(privkey, rsa, hash, &keybits, &is_public);
 	assert(success);
 	assert(keybits == 2048);
 	assert(!is_public);
 
-	success = l_key_get_info(key, rsa, L_CHECKSUM_NONE, &keybits,
+	success = l_key_get_info(privkey, rsa, L_CHECKSUM_NONE, &keybits,
 					&is_public);
 	assert(success);
 	assert(keybits == 2048);
@@ -554,14 +580,14 @@ static void test_key_crypto(const void *data)
 				sizeof(ciphertext), sizeof(decrypted));
 	assert(len < 0);
 
-	len = l_key_decrypt(key, rsa, hash, ciphertext, decrypted,
+	len = l_key_decrypt(privkey, rsa, hash, ciphertext, decrypted,
 				sizeof(ciphertext), sizeof(decrypted));
 	assert(len == (ssize_t)strlen(plaintext));
 	assert(strcmp(plaintext, (char *)decrypted) == 0);
 
 	/* Decrypt reference ciphertext */
 	memset(decrypted, 0, sizeof(decrypted));
-	len = l_key_decrypt(key, rsa, hash, reference_ciphertext, decrypted,
+	len = l_key_decrypt(privkey, rsa, hash, reference_ciphertext, decrypted,
 				sizeof(reference_ciphertext),
 				sizeof(decrypted));
 	assert(len == (ssize_t)strlen(plaintext));
@@ -571,7 +597,7 @@ static void test_key_crypto(const void *data)
 	memset(decrypted, 0, sizeof(decrypted));
 	memcpy(ciphertext, reference_ciphertext, sizeof(ciphertext));
 	ciphertext[0] = ciphertext[0] ^ (uint8_t)0xFF;
-	len = l_key_decrypt(key, rsa, hash, ciphertext, decrypted,
+	len = l_key_decrypt(privkey, rsa, hash, ciphertext, decrypted,
 				sizeof(ciphertext),
 				sizeof(decrypted));
 	assert(len < 0);
@@ -581,7 +607,7 @@ static void test_key_crypto(const void *data)
 				strlen(plaintext), sizeof(ciphertext));
 	assert(len < 0);
 
-	len = l_key_sign(key, rsa, hash, plaintext, ciphertext,
+	len = l_key_sign(privkey, rsa, hash, plaintext, ciphertext,
 				strlen(plaintext), sizeof(ciphertext));
 	assert(len == sizeof(ciphertext));
 
@@ -589,7 +615,7 @@ static void test_key_crypto(const void *data)
 				strlen(plaintext), sizeof(ciphertext));
 	assert(success);
 
-	success = l_key_verify(key, rsa, hash, plaintext, ciphertext,
+	success = l_key_verify(privkey, rsa, hash, plaintext, ciphertext,
 				strlen(plaintext), sizeof(ciphertext));
 	assert(success);
 
@@ -600,14 +626,12 @@ static void test_key_crypto(const void *data)
 
 	/* Corrupt signature */
 	ciphertext[42] = ciphertext[52] ^ (uint8_t)0xFF;
-	success = l_key_verify(key, rsa, hash, plaintext, ciphertext,
+	success = l_key_verify(privkey, rsa, hash, plaintext, ciphertext,
 				strlen(plaintext), sizeof(ciphertext));
 	assert(!success);
 
-	l_key_free(key);
+	l_key_free(privkey);
 	l_key_free(pubkey);
-	l_free(cert);
-	l_free(pubcert);
 }
 
 int main(int argc, char *argv[])
